package org.bdware.sc;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.bdware.sc.db.CMTables;
import org.bdware.sc.db.KeyValueDBUtil;
import org.bdware.sc.db.StatusRecorder;
import org.bdware.sc.units.MultiContractMeta;

import java.util.HashSet;
import java.util.Set;

public class MultiContractRecorder extends StatusRecorder<MultiContractMeta> {
    static final String dbName = CMTables.UnitContracts.toString();
    static final String prefix = "Multi_C_Meta_";
    private static final Logger LOGGER = LogManager.getLogger(MultiContractRecorder.class);

    public MultiContractRecorder(String dir) {
        super(dir, dbName, prefix);
        for (MultiContractMeta meta : getStatus().values()) {
            try {
                meta.initQueue();
                int lastExeSeq = Integer.parseInt(KeyValueDBUtil.instance
                        .getValue(CMTables.LastExeSeq.toString(), meta.getContractID()));
                meta.setLastExeSeq(lastExeSeq);
            } catch (Exception e) {
                LOGGER.error(e);
            }
        }
    }

    public MultiContractMeta getMultiContractMeta(String idOrNameOrDOI) {
        if (idOrNameOrDOI == null)
            return null;
        ContractMeta meta = ContractManager.instance.statusRecorder.getContractMeta(idOrNameOrDOI);
        return getMultiContractMeta(meta);
    }

    public MultiContractMeta getMultiContractMeta(ContractMeta meta) {
        if (meta == null)
            return null;
        return getStatus().get(meta.id);
    }

    public MultiContractMeta createIfNotExist(String contractID) {
        ContractMeta meta = ContractManager.instance.statusRecorder.createIfNotExist(contractID);
        MultiContractMeta ret = getMultiContractMeta(meta);
        if (null == ret) {
            LOGGER.info("requests don't contain contract " + contractID);
            ret = new MultiContractMeta(contractID);
            updateValue(ret);
        }
        return ret;
    }

    public void removeNotExist(ContractStatusRecorder statusRecorder) {
        Set<MultiContractMeta> toRemove = new HashSet<>();
        for (String key : getStatus().keySet()) {
            if (statusRecorder.getContractMeta(key) == null)
                toRemove.add(getStatus().get(key));
        }
        for (MultiContractMeta meta : toRemove)
            remove(meta);
    }
}
